@@ -11,7 +11,6 @@ from avocado.utils import path
 from avocado.utils import process
 
 from virttest import data_dir
-from virttest.compat_52lts import decode_to_text
 
 
 def _force_copy(src, dest):
@@ -588,8 +587,8 @@ class GnuSourceBuildHelper(object):
 
         :return: list of options accepted by configure script
         """
-        help_raw = decode_to_text(process.system_output(
-            '%s --help' % self.get_configure_path(), ignore_status=True))
+        help_raw = process.run('%s --help' % self.get_configure_path(),
+                               ignore_status=True).stdout_text
         help_output = help_raw.split("\n")
         option_list = []
         for line in help_output:
