@@ -1281,6 +1281,8 @@ def preprocess(test, params, env):
         suggest_mem = h.setup()
         if suggest_mem is not None:
             params['mem'] = suggest_mem
+        if not params.get("hugepage_path"):
+            params["hugepage_path"] = h.hugepage_path
         if vm_type == "libvirt":
             if libvirtd_inst is None:
                 libvirtd_inst = utils_libvirtd.Libvirtd()
@@ -1676,16 +1678,17 @@ def postprocess(test, params, env):
         logging.info("Sosreport for remote host: %s", sosreport_path)
     living_vms = [vm for vm in env.get_all_vms() if vm.is_alive()]
     # Close all monitor socket connections of living vm.
-    for vm in living_vms:
-        if hasattr(vm, "monitors"):
-            for m in vm.monitors:
-                try:
-                    m.close()
-                except Exception:
-                    pass
-        # Close the serial console session, as it'll help
-        # keeping the number of filedescriptors used by avocado-vt honest.
-        vm.cleanup_serial_console()
+    if not params.get_boolean("keep_env_vms", False):
+        for vm in living_vms:
+            if hasattr(vm, "monitors"):
+                for m in vm.monitors:
+                    try:
+                        m.close()
+                    except Exception:
+                        pass
+            # Close the serial console session, as it'll help
+            # keeping the number of filedescriptors used by avocado-vt honest.
+            vm.cleanup_serial_console()
 
     libvirtd_inst = None
     vm_type = params.get("vm_type")
