@@ -231,6 +231,20 @@ class Disk(base.TypedDeviceBase):
             setattr(new_one, key, value)
         return new_one
 
+    def get_backingstore_list(self):
+        """
+        Usage: get source file attribute from backingStore
+            test_disk = Disk()
+            backingstore_list = test_disk.get_backingstore_list()
+            source_file_list = [elem.find('source').get('file') or elem.find('source').get('name') for elem in backingstore_list]
+
+        :return: a disk backingstore list where each element is primitive virttest.element_tree._ElementInterface object
+        """
+        backingstore_list = []
+        for elem in self.xmltreefile.getiterator('backingStore'):
+            backingstore_list.append(elem)
+        return backingstore_list
+
     def new_drivermetadata(self, **dargs):
         """
         Return a new DriverMetadata instance and set properties from dargs
@@ -240,6 +254,30 @@ class Disk(base.TypedDeviceBase):
             setattr(new_one, key, value)
         return new_one
 
+    def get_all_backingstore(self):
+        """
+        Get all backingstore of a Disk object
+
+        :return: an ordered list of backingstore items
+        """
+
+        def _get_next_backingstore(elem):
+            """
+            Recursively get backingstore object
+
+            :param elem: root element to get backingstore object
+            """
+            if elem.xmltreefile.find('/backingStore') is None:
+                return
+            bs = elem.backingstore
+            backingstore_list.append(bs)
+            _get_next_backingstore(bs)
+
+        backingstore_list = []
+        _get_next_backingstore(self)
+
+        return backingstore_list
+
     # For convenience
     Address = librarian.get('address')
 
@@ -504,7 +542,7 @@ class Disk(base.TypedDeviceBase):
         source:
             nested xml of backingStore tag
         """
-        __slots__ = ('type', 'index', 'format', 'source')
+        __slots__ = ('type', 'index', 'format', 'source', 'backingstore')
 
         def __init__(self, virsh_instance=base.base.virsh):
             accessors.XMLAttribute('type', self,
@@ -524,6 +562,11 @@ class Disk(base.TypedDeviceBase):
                                      subclass=self.Source,
                                      subclass_dargs={
                                          'virsh_instance': virsh_instance})
+            accessors.XMLElementNest('backingstore', self, parent_xpath='/',
+                                     tag_name='backingStore',
+                                     subclass=Disk.BackingStore,
+                                     subclass_dargs={
+                                         'virsh_instance': virsh_instance})
 
             super(self.__class__, self).__init__(virsh_instance=virsh_instance)
             self.xml = '<backingStore/>'
@@ -554,9 +597,12 @@ class Disk(base.TypedDeviceBase):
                 string, attribute of backingStore/source tag
             """
 
-            __slots__ = ('dev', 'protocol', 'name', 'host', 'file', 'auth')
+            __slots__ = ('attrs', 'dev', 'protocol', 'name', 'host', 'file', 'auth')
 
             def __init__(self, virsh_instance=base.base.virsh):
+                accessors.XMLElementDict('attrs', self,
+                                         parent_xpath='/',
+                                         tag_name='source')
                 accessors.XMLAttribute('dev', self,
                                        parent_xpath='/',
                                        tag_name='source',
