@@ -480,6 +480,10 @@ class LibvirtQemuConfig(LibvirtConfigCommon):
         'swtpm_user': 'string',
         'swtpm_group': 'string',
         'capability_filters': 'list',
+        'nbd_tls_x509_secret_uuid': 'string',
+        'backup_tls_x509_cert_dir': 'string',
+        'backup_tls_x509_verify': 'boolean',
+        'backup_tls_x509_secret_uuid': 'string',
     }
 
 
@@ -745,6 +749,7 @@ def get_conf_obj(config_type):
         'virtproxyd': VirtProxydConfig,
         'virtnetworkd': VirtNetworkdConfig,
         'virtstoraged': VirtStoragedConfig,
+        'virtsecretd': VirtSecretdConfig,
         'virtinterfaced': VirtInterfacedConfig,
         'virtnodedevd': VirtNodedevdConfig,
         'virtnwfilterd': VirtNwfilterdConfig
