@@ -34,8 +34,10 @@ from aexpect import remote
 from avocado.utils import path
 from avocado.utils import process
 
-from . import propcan
-from . import utils_misc
+from virttest import propcan
+from virttest import utils_misc
+from virttest import utils_split_daemons
+from virttest import utils_config
 
 
 # list of symbol names NOT to wrap as Virtadmin class methods
@@ -702,6 +704,32 @@ def command(cmd, **dargs):
     return ret
 
 
+def check_server_name(server_name="virtproxyd"):
+    """
+    Determine the server name under different daemon mode.
+
+    :param server_name: name of the managed server
+    :return: name of the managed server
+    """
+    if not utils_split_daemons.is_modular_daemon():
+        server_name = "libvirtd"
+    return server_name
+
+
+def managed_daemon_config(conf_type="virtproxyd"):
+    """
+    Determine different daemon config under different daemon mode.
+
+    :param conf_type: The configuration type to get
+        For example, "libvirtd" or "virtqemud"
+    :return: utils_config.LibvirtConfigCommon object
+    """
+    if not utils_split_daemons.is_modular_daemon():
+        conf_type = "libvirtd"
+    config = utils_config.get_conf_obj(conf_type)
+    return config
+
+
 def cd(dir_path, options="", **dargs):
     """
     Run cd command in virsh interactive session.
@@ -795,6 +823,17 @@ def srv_clients_info(server_name, **dargs):
     return command(cmd, **dargs)
 
 
+def server_update_tls(server_name, **dargs):
+    """
+    Run server-update-tls in virt-admin session
+    :param server_name: name of the server
+    :param dargs: standardized virt-admin function API keywords.
+    :return: CmdResult object
+    """
+    cmd = "server-update-tls %s " % server_name
+    return command(cmd, **dargs)
+
+
 def srv_clients_list(server_name, **dargs):
     """
     Run srv-clients-list in virt-admin session
